#include <stdio.h>
#include <LPC214x.H>                       /* LPC214x definitions */
#include "lcd.h"
#include "type.h"
#include "i2c.h"
#include "adc.h"
#include "74HC595.h"
#include "uart.h"
#include "vcom.h"
#include "lpc2000_spi.h"
#include "ps2kbd.h"
#include "spi.h"
#include "ext_int.h"
#include "test.h"
#include "usbapi.h"
#include "dac.h"
#include "led.h"



/**
**************************************************************************

****1111
  Function Name : wait()

  Description :This function suspends the tasks for specified ticks.	

  Input :  ticks:no of ticks in multiple of 1 usec
            task: task to be suspended

  Output : void

  Note :
*******************************************************************************
*/

void wait(int count)
{
  int j=0,i=0;

  for(j=0;j<count;j++)
  {
    /* At 60Mhz, the below loop introduces
    delay of 10 us */
    for(i=0;i<35;i++);
  }
}

/**
*****************************************************************
	Function Name : DAC_test()

	Description   : 

	Input         : 

	Output        : 

	Note          :
*****************************************************************
*/


void DAC_test(void)
{
	uint32_t i=0,j=0;
	uint32_t k=0;

  for(k=0;k<10000;k++)
  {
	DACR = (i << 6) | DAC_BIAS;
	i++;
	for(j = 1000; j > 1; j--);
	if ( i == 1024 )
	{
	  i = 0;
	}
  }
}

/**
*****************************************************************
	Function Name : hardware_init()

	Description   : 

	Input         : 

	Output        : 

	Note          :
*****************************************************************
*/

void hardware_init(void)
{
  SCS = 3;    //sets port 1 and 0 to fast mode (not compatable with previous LPCs)
  

  init_lcd();
  init_ext_interrupt();   // initialize the external interrup
  init_keyboard();
  init_uart0();
  init_uart1();
  DACInit();
  init_adc0();
  i2c_lpc_init(I2C_SPEED_100);		// Initialize I2C
  VCOM_init();
}



int main (void) 
{
  U8 ch=0;
  test_led();
  hardware_init();
  test_sd_card();
  test_i2c_at24c256_flash();	   // Test I2C EEPROM by wrinting some data at some address and reading the same from same address
  
  DAC_test();
  lcd_clear();

  while (1)  
  {
  	if(get_USB_enumeration_status())
	{
		if(VCOM_read(sizeof(ch),&ch))
		{
			test_usb();
		}
	}
		
    if(uart0Receive(&ch, sizeof(ch)) != -1)
    {
      test_uart0();
    }
    if(uart1Receive(&ch,sizeof(ch)) != -1)
    {
      test_uart1();
    }
	process_kbd();
	process_adc();	// Read ADC value and display it on first line of LCD
  
    }
}
