#include <stdio.h>
#include "lcd.h"
#include "type.h"
#include "i2c.h"
#include "adc.h"
#include "74HC595.h"
#include "uart.h"
#include "vcom.h"
#include "lpc2000_spi.h"
#include "ps2kbd.h"

#define GET_SCALED_LED_MASK_BUF_INDEX(val) (val/114)

const U8 led_mask_buf[] = {0,1,3,7,0xF,0x1F,0x3F,0x7F,0xFF};

#define UART0_TEXT "\n\r Testing UART0 NGX's BlueBoard \n\r BlueBoard Revision : 1 \n\r Firmware Version: 1 \n\r For more information on BlueBoard visit www.ngxtechnologies.com"
#define UART1_TEXT "\n\r Testing UART1 NGX's BlueBoard \n\r BlueBoard Revision : 1 \n\r Firmware Version: 1 \n\r For more information on BlueBoard visit www.ngxtechnologies.com"
#define USB_TEXT "\n\r Testing USB NGX's BlueBoard \n\r BlueBoard Revision : 1 \n\r Firmware Version: 1 \n\r For more information on BlueBoard visit www.ngxtechnologies.com"

U8 init_sdcard(void)
{
 if(if_initInterface(0)==0)
 return 1;
 else
 return 0;
}

/**
*******************************************************************
	Funtion Name : test_i2c_at24c256_flash()

	Description  : 

	Input        : 

	Output       : None

	Note         :
*****************************************************************
*/
void test_i2c_at24c256_flash(void)
{
  unsigned char i=0;
  unsigned char status=1;
  unsigned char buf[128];

  for(i=0;i<sizeof(buf);i++)
    buf[i] = i;
  
  m24xx_write(EEPROM_24XX256, 0 , 0 , (char *)buf , sizeof(buf));

  for(i=0;i<sizeof(buf);i++)
    buf[i] = 0;

  m24xx_read(EEPROM_24XX256, 0 , 0 , (char *)buf , sizeof(buf));

  for(i=0;i<sizeof(buf);i++)
  {
    if(buf[i] != i)
    {
      status = 0;
      break;
    }    
  }

  if(status == 1)
  {
    lcd_putstring(LINE2,"I2C FLASH - PASS");
  }
  else
  {
	lcd_putstring(LINE2,"I2C FLASH - FAIL");
  }

}

/**
*******************************************************************
	Funtion Name : test_sd_card()

	Description  : 

	Input        : 

	Output       : None

	Note         :
*****************************************************************
*/
void test_sd_card(void)
{
  if(init_sdcard())
  {
    lcd_clear();
    lcd_putstring(LINE1,"SD CARD - PASS");
  }
  else
  {
    lcd_clear();
    lcd_putstring(LINE1,"SD CARD - FAIL");
  }
}

/**
********************************************************************************************
	Function Name :	is_valid_ascii()

	Description :
	
	Input :	Void

	Output : Void

	Note :
**********************************************************************************************
*/
U8 is_valid_ascii(S8 key)
{
  U8 retval = FALSE;
  
  if(key >= ' ' && key <= '~')
    retval = TRUE;

  return retval;
}

/**
*******************************************************************************
  Function Name : test_uart0()

  Description :	 

  Input :	

  Output :	none

  Note : 
*******************************************************************************
*/
void test_uart0(void)
{
  U8 buffer[] = UART0_TEXT;
  
  uart0Write((char*)buffer, sizeof(buffer));
}

/**
*******************************************************************************
  Function Name : test_uart1()

  Description :	 

  Input :	

  Output :	none

  Note : 
*******************************************************************************
*/
void test_uart1(void)
{
  U8 buffer[] = UART1_TEXT;

  uart1Write((char*)buffer, sizeof(buffer));
}

/**
********************************************************************************************
	Function Name :	process_lcd()

	Description :
	
	Input :	Void

	Output : Void

	Note :
**********************************************************************************************
*/
void process_kbd(void)
{
	S8 key;
  	static U8 position = 0;

  	if(keyhit())	
  	{
  		key = xgetkey();
  		if(is_valid_ascii(key))
  		{
    		lcd_gotoxy(LINE2, position);
    		lcd_putchar(key);

    		if(position == MAX_CHAR_IN_ONE_LINE)
    		{
      			lcd_clear();
      			position = 0;
			}		
    	position++;
		}
	}
}

/**
*******************************************************************
	Funtion Name : test_usb()

	Description  : 

	Input        : 

	Output       : None

	Note         :
*****************************************************************
*/
void test_usb(void)
{
  U8 buffer[] = USB_TEXT;

  while((VCOM_write(sizeof(buffer), buffer)) != sizeof(buffer));
}

/**
********************************************************************************************
	Function Name :	process_adc()

	Description :
	
	Input :	Void

	Output : Void

	Note :
**********************************************************************************************
*/
void process_adc(void)
{
  	
	unsigned short adc_value = 0;
  	unsigned char buf[16] = {0};
  	adc_value = adc_read(ADC0, CHANNEL_3);	
  	sprintf((char *)buf, "ADC:%4d ", adc_value);
  	lcd_putstring(LINE1, (char *)buf);
}
